import path from 'path';
import Koa, { Context, DefaultState } from 'koa';
// 路由
import router from './router';
// 美化控制台
// import consola from 'consola/dist/consola';
// 解析post的body体
import bodyParser from 'koa-bodyparser';
// 部分安全相关
import helmet from 'koa-helmet';
// 静态服务
import koaStatic from 'koa-static';
// 请求转发
import koaProxy from 'koa-proxies';
// 导入配置文件
import config from '@config';
// 导入中间件
// import redisMiddleware from './middlewares/redis';
// import limitMiddleware from './middlewares/limit';
// import logsMiddleware from './middlewares/logs';
// import responseMiddleware from './middlewares/response';
// import sessionMiddleware from './middlewares/session';
// // import authMiddleware from './middlewares/auth';
// import localDbMiddleware from './middlewares/localDb';
import middlewares from './middlewares';
import server from '@utils/server';
import Plugins from '@/plugins';

const plugins = Plugins();
const app = new Koa<DefaultState, Context>();

// 为了获取到真实的ip
// https://github.com/koajs/koa/issues/599
app.proxy = true;

/* 中间件 */
// 安全防护
app.use(helmet());
// 静态资源
app.use(koaStatic(path.join(__dirname, config.staticPath)));
// 返回体格式
// app.use(responseMiddleware());
// post params解析
app.use(bodyParser({
  ...config.bodyParser
}));
app.use(middlewares(app, plugins));
// 日志
// app.use(logsMiddleware(plugins.logger.middleware));

// // 本地嵌入式数据库
// app.use(localDbMiddleware(plugins.db));
// // redis
// app.use(redisMiddleware({
//   redis: plugins.redis,
//   memoryCache: plugins.memoryCache
// }));
// // 限流
// app.use(limitMiddleware());
// // session
// app.use(sessionMiddleware(app));
// auth
// app.use(authMiddleware())

// 请求转发
const { run, ...proxyOptions } = config.proxy;
if (run) {
  app.use(koaProxy('/proxy', { ...proxyOptions }));
}

app.on('error', (err, ctx: Context) => {
  // ctx.$response(err, 'error', false);
  const errMsg: string = err.message || '服务出错';
  console.error('app >>>', err);
  // console.log(ctx);
  plugins.logger.formatLog('error', errMsg, ctx);
});

// 注册路由
app.use(router.routes()).use(router.allowedMethods());

server(app);
