import { Context } from 'koa';
import services from '../services';
import CODE from '../utils/code';

interface InterfaceSession {
  user?: string
}

export interface InterfaceRegisterParams {
  username: string
  password: string
}

const register = async (ctx: Context): Promise<void> => {
  const { username, password } = ctx.request.body;
  if (!username) {
    ctx.body = ctx.$response(null, '用户名不能为空', false, CODE.MISS_PARAMS);
    return;
  }
  if (!password) {
    ctx.body = ctx.$response(null, '密码不能为空', false, CODE.MISS_PARAMS);
    return;
  }
  const params: InterfaceRegisterParams = {
    username,
    password
  };
  const data = await services.user.register(ctx, params);
  ctx.body = ctx.$response(data.data, data.message, data.success, data.code);
};

const login = async (ctx: Context): Promise<void> => {
  const { username, password } = ctx.request.body;
  if (!username) {
    ctx.body = ctx.$response(null, '用户名不能为空', false, CODE.MISS_PARAMS);
    return;
  }
  if (!password) {
    ctx.body = ctx.$response(null, '密码不能为空', false, CODE.MISS_PARAMS);
    return;
  }
  const params: InterfaceRegisterParams = {
    username,
    password
  };
  const data = await services.user.login(ctx, params);
  ctx.body = ctx.$response(data.data, data.message, data.success, data.code);
};

const logout = async (ctx: Context): Promise<void> => {
  if (ctx.session) {
    ctx.session.user = null;
  }
  ctx.body = ctx.$response(null, '退出成功');
};

const userInfo = async (ctx: Context): Promise<void> => {
  if (!ctx.session || !ctx.session.user ) {
    ctx.body = ctx.$response(null, '未登录', false, CODE.NOT_LOGIN);
    return;
  }
  const session: InterfaceSession = ctx.session.toJSON();
  ctx.body = ctx.$response(session.user);
};

export default {
  register,
  login,
  logout,
  userInfo
};
