import { Context, DefaultState } from 'koa';
import Router from 'koa-router';
// 解析 multipart/form-data
import multer from '@koa/multer';
import controllers from '@controllers';
import config from '@config';
const upload = multer({ ...config.uploadFile });

// api 路由
const apiRouter = new Router<DefaultState, Context>();
apiRouter.get('/code', controllers.code);
apiRouter.put('/register', controllers.user.register);
apiRouter.post('/login', controllers.user.login);
apiRouter.get('/logout', controllers.user.logout);
apiRouter.get('/user-info', controllers.user.userInfo);
apiRouter.put('/document', controllers.document.putDocument);
apiRouter.get('/document/:id', controllers.document.getDocument);
apiRouter.get('/document', controllers.document.getDocumentList);
apiRouter.delete('/document/:id', controllers.document.deleteDocument);
apiRouter.post('/document/:id', controllers.document.updateDocument);
apiRouter.put('/tag', controllers.document.putTag);
apiRouter.post('/tag/:id', controllers.document.updateTag);
apiRouter.get('/tag', controllers.document.getTagList);
apiRouter.delete('/tag/:id', controllers.document.deleteTag);
apiRouter.get('/upload-init', controllers.bigFileUpload.initUpload);
apiRouter.post('/upload-part',
  upload.single('partFile'),
  controllers.bigFileUpload.uploadPart
);
apiRouter.get('/upload-finish', controllers.bigFileUpload.finishUpload);

// 主路由
const router = new Router<DefaultState, Context>();
router.get('/', controllers.home);
router.get('/ws', controllers.ws);
router.get('/upload', controllers.upload);
router.post('/register', controllers.register);

// api路由以/api前缀
router.use('/api', apiRouter.routes(), apiRouter.allowedMethods());

export default router;
