import { Context } from 'koa';
import { InterfaceResponseData } from '@utils/response';
import { InterfaceRegisterParams } from '@/controllers/user';
import CODE from '@utils/code';

const register = async (
  ctx: Context,
  params: InterfaceRegisterParams
) :Promise<InterfaceResponseData> => {
  const reData: InterfaceResponseData = {
    data: null,
    message: '',
    success: false,
    code: 400000
  };
  return new Promise(resolve => {
    ctx.$db.user.findOne({ username: params.username }, (err, findUser) => {
      if (err) {
        ctx.$log.error(err);
        ctx.status = 500;
        resolve(reData);
        return;
      }
      if (findUser) {
        reData.message = '用户名已存在';
        reData.code = CODE.EXIST_PARAMS;
        resolve(reData);
        return;
      }
      ctx.$db.user.insert(params, (err, saveUser) => {
        if (err) {
          ctx.$log.error(err);
          ctx.status = 500;
          resolve(reData);
          return;
        }
        reData.data = saveUser;
        reData.message = '注册成功';
        reData.success = true;
        reData.code = CODE.SUCCESS;
        if (ctx.session) {
          ctx.session.user = saveUser;
        }
        resolve(reData);
      });
    });
  });
};

const login = async (
  ctx: Context,
  params: InterfaceRegisterParams
) :Promise<InterfaceResponseData> => {
  const reData: InterfaceResponseData = {
    data: null,
    message: '',
    success: false,
    code: 400000
  };
  return new Promise(resolve => {
    ctx.$db.user.findOne({ username: params.username }, (err, findUser) => {
      if (err) {
        ctx.$log.error(err);
        ctx.status = 500;
        resolve(reData);
        return;
      }
      if (!findUser) {
        reData.message = '用户名或密码错误';
        reData.code = CODE.MISS_PARAMS;
        resolve(reData);
        return;
      }
      if (params.password === findUser.password) {
        if (ctx.session) {
          ctx.session.user = findUser;
        }
        reData.data = findUser;
        reData.message = '登录成功';
        reData.success = true;
        reData.code = CODE.SUCCESS;
        resolve(reData);
      } else {
        reData.message = '用户名或密码错误';
        reData.code = CODE.MISS_PARAMS;
        resolve(reData);
      }
    });
  });
};

export default {
  register,
  login
};
