import Koa, { DefaultState, Context } from 'koa';
import WebSocket from 'ws';
import consola from 'consola';
import config from '@config';
import { getLocalIp } from '@utils/getLocalIp';
import { Server } from 'http';

export default (app: Koa<DefaultState, Context>): Server => {
  const port = config.port;
  const ip = getLocalIp();
  const server = app.listen(port, () => {
    process.title = 'node服务';
    consola.success(`server is started at: http://localhost:${port}`);
    consola.success(`server is started at: http://${ip}:${port}`);
  });
  if (config.websocket.run) {
    const wsServer = new WebSocket.Server({ server }, () => {
      consola.success(`websocket server is started at: ws://localhost:${port}`);
      consola.success(`websocket server is started at: ws://${ip}:${port}`);
    });
    wsServer.on('error', (err) => {
      consola.error('websocket error >>>', err);
    }).on('close', () => {
      consola.info('websocket is closed');
    });
    wsServer.on('connection', ws => {
      console.log('ws is connectioned');
      ws.on('message', msg => {
        console.log(msg);
        const obj = {
          msg,
          code: 200
        };
        ws.send(JSON.stringify(obj), err => {
          if (err) {
            consola.error('websocket send >>>', err);
          }
        });
      });

    });
  }
  return server;
};
